'' System Version 2.001
'' OS_initialization Changes:
''
''   2006/09/24 - Moved from the main program to a separate object.  Added a parameter
''                for the initial display message.

'' This is the initialization routine for the Propeller Operating System.
'' It is part of the display specific portion of the OS mostly because of
'' the I/O pin definitions referenced.  Please adjust these as needed.

CON
' dspPinGroup  = 12                            ' Display pin group (typical for TV)
  dspPinGroup  = 16                            ' Display pin group (typical for VGA)
  ps2KeyData   = 26                            ' PS/2 Keyboard Clock pin
  ps2KeyClock  = 27                            ' PS/2 Keyboard Data pin
  
OBJ
  ldr     : "OS_loader"                        ' SPIN program loader and support routines
  ldrInit : "OS_loaderInit"                    '  Initialization and assembly portion
  dsp     : "OS_dspText"                       ' Display text driver modified for OS use
  dspInit : "OS_dspTextInit"                   '  Initialization and assembly portion
  key     : "OS_keyboard"                      ' PS/2 keyboard driver modified for OS use
  keyInit : "OS_keyboardInit"                  '  Initialization and assembly portion
  files   : "OS_fileSystem"                    ' File system support

PUB start(s) | t, i, p
'' Operating System initialization.  Only check for keyboard and presence of boot EEPROM
'' after the first time.  A marker is placed in high memory (at ldr#initMarker) to indicate
'' that initialization has already occurred.  This is erased during the boot process.
'' The parameter is the identification string to use once the display is initialized.
  if long[ldr#initMarker] <> ldr#uniqueMark                ' Check for a valid initialization
    repeat i from 0 to 7
      if i <> cogid                                        ' Stop all cogs except this one
        cogstop(i)
    long[ldr#memPtr]    := ldr#endFree                     ' Start at end of HUB memory
    p := ldr.allocatePRI(ldr#systemSize)                   ' Allocate system work area
    long[ldr#systemPtr] := p
    long[p+ldr#sys_cog] := |<cogid                         ' Mark current cog in use
    byte[p+ldr#sys_cogMain] := cogid + 1
    long[p+ldr#sys_random] := ldr#initMarker               ' Initialize random seed
    wordfill(p+ldr#sys_eeproms,0,ldr#maxPinPairs)          ' Clear EEPROMs bitmap table
    long[ldr#videoPtr]  := ldr.allocatePRI(ldr#videoSize)  ' Allocate work area for video
    long[ldr#screenPtr] := ldr.allocatePRI(ldr#screenSize) ' Allocate screen buffer
    long[ldr#colorPtr]  := ldr.allocatePRI(ldr#colorSize)  ' Allocate screen color buffer
    long[ldr#fontPtr]   := ldr.allocatePRI(ldr#fontSize)   ' Allocate font table
    long[ldr#cursorPtr] := ldr.allocatePRI(ldr#cursorSize) ' Allocate cursor control block
    if not dspInit.start(dspPinGroup)                      ' Now start up the display driver
      abort false
    dsp.str(s)                                             ' Display a "sign on" string
    long[ldr#keybdPtr]  := ldr.allocatePRI(ldr#keybdSize)  ' Allocate work area for keyboard
    if not keyInit.start(ps2KeyData,ps2KeyClock)           ' Now start up the keyboard driver
      abort false
    if not ldr.start(ldrInit.getEntryPoint)                ' Now start up the I2C driver
      abort false                                         
    long[ldr#userPtr] := ldr#noSuchAddr
    long[ldr#initMarker] := ldr#uniqueMark                 ' Basic initialization complete
    waitcnt(clkfreq * 2 + cnt)                             ' Wait for keyboard to initialize
    dsp.out(dsp#Cr)
  if not key.present
    dsp.str(string("--> keyboard not present"))
    long[ldr#initMarker] := 0
    abort false
  if (\files.updateBitmapEEPROMs(ldr#i2cBootSCL) & $4000) == 0  ' Check boot EEPROM present
    dsp.str(string("--> boot eeprom not accessable"))
    long[ldr#initMarker] := 0
    abort false
  