'' System Version 2.001
'' uALFAT_Interface Changes:
''
''   2006/09/24 - Initial version with change dates

'' This object provides an SPI interface to the GHI Electronics
'' uALFAT-SD MMC/SD card interface processor.

CON
  uALFAT_SSel   = 0                                 ' uALFAT SPI Select
  uALFAT_MoSi   = 1                                 ' uALFAT Master out, Slave in
  uALFAT_MiSo   = 2                                 ' uALFAT Master in, Slave out
  uALFAT_SClk   = 3                                 ' uALFAT SPI Clock
  uALFAT_Bsy    = 4                                 ' uALFAT Output Busy (high true)
  uALFAT_Rdy    = 5                                 ' uALFAT Input Ready (high true)
  uALFAT_Rst    = 6                                 ' uALFAT Reset (low true)
  
  uALFAT_EndDir = $4D                               ' End of directory information
  uALFAT_NDT    = $FF                               ' SPI No Data
  uALFAT_HDT    = $FE                               ' SPI Half Data

OBJ
  ldr : "OS_loader"                                 ' OS loader and I2C routines
  dsp : "OS_dspText"                                ' OS display driver interface

PUB start                                           '' Set up the I/O pins
  dira[uALFAT_Rdy] ~                                ' Set some pins to inputs
  dira[uALFAT_Bsy] ~
  dira[uALFAT_MiSo] ~                               ' SPI Master in, Slave out
  dira[uALFAT_MoSi] ~~                              ' SPI Master out, Slave in
  outa[uALFAT_SSel] ~~                              ' Set both mode bits for SPI
  dira[uALFAT_SSel] ~~
  outa[uALFAT_SClk] ~~
  dira[uALFAT_SClk] ~~
  outa[uALFAT_Rst] ~                                ' Take reset LOW then HIGH
  dira[uALFAT_Rst] ~~
  dira[uALFAT_Rst] ~
  
PUB stop                                            '' Make sure the pins are left as inputs
  dira[uALFAT_MoSi] ~
  dira[uALFAT_SSel] ~
  dira[uALFAT_SClk] ~

PUB connectCard(ms) | e                             '' Prepare to access an SD/MMC card
  sendStr(string("I",13))                           ' Attach to the media
  checkErrorCode(ms,0)

PUB listDirectory(ms) | e, d, a, s, n0, n1, n2, n3  '' Display the current directory
  sendStr(string("@",13))                           ' Initialize the directory listing
  checkErrorCode(ms)
  dsp.str(string("File Name    Attr     File Size",dsp#Cr))
  repeat
    sendStr(string("N",13))                         ' Get the next file entry
    if checkErrorCode(ms,uALFAT_EndDir)
      quit
    getFileName(ms,@n0)
    checkNext(ms," ")
    a := getHex(ms,2)                               ' Get file attribute
    checkNext(ms," ")
    s := getHex(ms,8)                               ' Get file size
    checkNext(ms,13)
    checkErrorCode(ms,0)
    dsp.str(@n0)                                    ' Display the information
    repeat 13 - strsize(@n0)
      dsp.out(" ")
    dsp.bin(a,8)
    dsp.out(" ")
    dsp.dec(s)
    dsp.str(string(" Bytes",dsp#Cr))
  sendStr(string("K",13))                           ' Get media statistics
  checkErrorCode(ms,0)
  a := getHexLong(ms)                               ' Get media size
  checkNext(ms," ")
  s := getHexLong(ms)                               ' Get free size
  checkNext(ms,13)
  checkErrorCode(ms,0)
  dsp.dec(a)
  dsp.str(string(" Total Size, "))
  dsp.dec(s)
  dsp.str(string(" Bytes Free",dsp#Cr))

PUB changeDir(ms, s) : e                            '' Change to specified subdirectory
  sendStr(string("A "))
  sendStr(s)
  sendChar(13)
  checkErrorCode(ms,0)

PUB openFile(ms, s, mode) : e                       '' Open specified file in the specified mode
  sendStr(string("OF 0"))                           ' Send command and use handle zero
  sendChar(mode)                                    ' Send file mode ("R", "W", "A")
  sendChar(">")
  sendStr(s)                                        ' Send file name
  sendChar(13)
  checkErrorCode(ms,0)

PUB closeFile(ms) | e                               '' Close current file
  sendStr(string("C 0",13))                         ' Send command and use handle zero
  checkErrorCode(ms,0)
  
PUB flushFile(ms) | e                               '' Flush current file
  sendStr(string("F 0",13))                         ' Send command and use handle zero
  checkErrorCode(ms,0)
  
PUB readFile(ms, bufAdr, count) | e, t
  sendStr(string("R 0",$7F,">"))
  sendHex(count,8)
  sendChar(13)
  checkErrorCode(ms,0)
  repeat count
    if (t := recvChar(ms)) == -1
      abort t
    byte[bufAdr++] := t
  if getHexLong(ms) <> count
    abort -1
  checkNext(ms,13)
  checkErrorCode(ms,0)

PUB writeFile(ms, bufAdr, count) | e, t
  sendStr(string("W 0>"))
  sendHex(count,8)
  sendChar(13)
  checkErrorCode(ms,0)
  repeat count
    sendChar(byte[bufAdr++])
  if getHexLong(ms) <> count
    abort -1
  checkNext(ms,13)
  checkErrorCode(ms,0)

PUB deleteFile(ms, nameBuf) | e
  sendStr(string("D "))
  sendStr(nameBuf)
  sendChar(13)
  checkErrorCode(ms,0)

PUB getFileName(ms, nameBuf) | i, name, name2, ext  '' Receive a fixed width 8.3 file name
  getTextField(ms,@name,8)                          ''  suppress trailing blanks and make
  checkNext(ms,".")                                 ''  into a single zero-terminated string
  getTextField(ms,@ext,3)
  i := 0
  repeat while i < 8 and name[i] <> " "
    byte[nameBuf++] := name[i]
  byte[nameBuf++] := "."
  i := 0
  repeat while i < 3 and ext[i] <> " "
    byte[nameBuf++] := ext[i]
  byte[nameBuf] := 0

PUB getHexLong(ms)                                  '' Receive a hex long prefixed by "$"
  checkNext(ms,"$")
  return getHex(ms,8)

PUB getHexWord(ms)                                  '' Receive a hex word prefixed by "$"
  checkNext(ms,"$")
  return getHex(ms,4)

PUB getHexByte(ms)                                  '' Receive a hex byte prefixed by "$"
  checkNext(ms,"$")
  return getHex(ms,2)

PUB getTextField(ms, bufAddr, size) | t             '' Receive a fixed size text field
  repeat size
    if (t := recvChar(ms)) == -1
      abort t
    byte[bufAddr++] := t

PUB checkErrorCode(ms, code))                       '' Make sure error code is zero
  if (result := getErrorCode(ms)) <> 0              ' Allow optional specific code
    if result <> code
      abort $100 | result

PUB getErrorCode(ms) : err                          '' Receive an error code ("!00",13)
  checkNext(ms,"!")
  err := getHex(ms,2)
  checkNext(ms,13)

PUB checkNext(ms,char)                              '' Check the next received character
  if (result := recvChar(ms)) == -1 or result <> char
    abort result

PUB getHex(ms,size) : hex | t                       '' Receive a fixed width hex value
  repeat size
    if (t := recvChar(ms)) == -1
      abort -1
    case t
      "0".."9": hex := (hex << 4) | (t - "0")
      "A".."F": hex := (hex << 4) | (t - "A" + 10)
      "a".."f": hex := (hex << 4) | (t - "a" + 10)
      other: abort t

PUB sendChar(c)                                     '' Send a character to the uALFAT
  repeat while ina[uALFAT_Bsy]                      ' Wait for uALFAT not Busy
  if c == uALFAT_NDT or c == uALFAT_HDT
    spiTransfer(uALFAT_HDT)                         ' Handle the case of uALFAT_HDT
    if c == uALFAT_HDT                              '  and uALFAT_NDT values
      spiTransfer(uALFAT_HDT)
    else
      spiTransfer(0)                                ' Use zero for uALFAT_NDT
  else
    spiTransfer(c)                                  ' Send the actual character

PUB sendStr(s) | c                                  '' Send a string to the uALFAT
  result := true                                    ' Return true if successful
  repeat while (c := byte[s++])
    if not (result := sendChar(c))
      quit

PUB sendHex(v, w) | s, t                            '' Send a fixed width hex value
  result := true                                    ' Return true if successful
  repeat s from (w - 1) * 4 to 0 step 4
    if (t := ((v >> s) & $F)) < 10
      result := sendChar(t + "0")                   ' "0".."9"
    else
      result := sendChar(t - 10 + "A")              ' "A".."F"
    if not result
      quit
  
PUB recvChar(ms) : c | t, d                         '' Receive a character from the uALFAT
  c := -1
  d := clkfreq / 1000                               ' Wait for up to ms
  t := cnt
  repeat until ina[uALFAT_Rdy] == 1 or (cnt - t) / d > ms
  if ina[uALFAT_Rdy] == 1                           ' Get a character if ready
    c := spiTransfer(uALFAT_NDT)

PRI spiTransfer(c)
  outa[SPIssel]~
  repeat 8
    outa[SPIsck]~
    outa[SPImosi] := (c <<= 1) >> 8
    result := (result << 1) | ina[SPImiso]
    outa[SPIsck]~~
  outa[SPIssel]~~
